/* ************************************************************************
*  file:  plrtoascii.c                                  Part of CircleMud *
*  Copyright (C) 1990, 1991 - see 'license.doc' for complete information. *
*  All Rights Reserved                                                    *
************************************************************************* */

#include "../structs.h"
#include "../utils.h"
#include "../db.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>

void sprintbits(long vektor,char *outstring);
void store_to_char(struct char_file_u * st, struct char_data * ch);
void save_char(struct char_data * ch, sh_int load_room);
void kill_ems(char *str);
char *str_dup(const char *source);
const char *pc_race_types[] = {
  "Human",
  "Dwarf",
  "Elf",
  "Ork",
  "Troll",
  "\n"
};

#define PLR_INDEX_FILE_N "lib/etc/pfiles/plr_index"
#define PLR_PREFIX_N	 "lib/etc/pfiles"
#define SLASH           "/"
/* change these if you want to put all files in the same directory (or if
   you just like big file names
*/
#define PLR_SUFFIX	""

void convert(char *filename)
{
  FILE *fl;
  FILE *index_file;
  struct char_file_u *player = NULL;
  struct char_data *ch = NULL;
  long size, recs;
  char bits[64];

  if (!(fl = fopen(filename, "r+")))
  {
    perror("error opening playerfile");
    exit(1);
  }
  if(!(index_file = fopen(PLR_INDEX_FILE_N, "w+b")))
  {
    perror("SYSERR:  Could not write player index file");
    exit(1);
  }

  fseek(fl, 0L, SEEK_END);
  size = ftell(fl);
  rewind(fl);
  if (size % sizeof(struct char_file_u))
    fprintf(stderr, "\aWARNING:  PLAYERFILE IS PROBABLY CORRUPT!\n");
  recs = size / sizeof(struct char_file_u);
  if (recs)
    printf("%ld players in database.", recs);

  for (;;)
  {
    player = new char_file_u;
    fread(player, sizeof(struct char_file_u), 1, fl);
    if (feof(fl))
    {
      fclose(fl);
      fprintf(index_file, "~\n");
  	  fclose(index_file);
      exit(0);
    }
    ch = new char_data;
    ch->player_specials = new player_special_data;
    memset((char *) ch, 0, sizeof(struct char_data));
    //ch->in_room = NOWHERE;
	//GET_PFILEPOS(ch) = -1;
	//GET_WAS_IN(ch) = NOWHERE;
	//GET_POS(ch) = POS_STANDING;
    store_to_char(player,ch);
    // Validate a Valid player here
    printf("PName :%s\r\n",GET_NAME(ch));
    save_char(ch,GET_LOADROOM(ch));
    //fprintf(index_file, "%ld %s\n",GET_IDNUM(ch),GET_NAME(ch));
    sprintbits(PLR_FLAGS(ch), bits);
    fprintf(index_file, "%ld %s %d %s %ld\n",GET_IDNUM(ch),GET_NAME(ch),GET_LEVEL(ch),bits,ch->player.time.logon);

  }
}


int main(int argc, char **argv)
{
  if (argc != 2)
    printf("Usage: %s playerfile-name\n", argv[0]);
  else
    convert(argv[1]);

  return 0;
}

void save_char(struct char_data * ch, sh_int load_room)
{
  FILE *fl;
  char outname[40], bits[127], buf[MAX_STRING_LENGTH];
  int i, id, save_index = FALSE;
  struct affected_type *aff, tmp_aff[MAX_AFFECT];
  struct obj_data *char_eq[NUM_WEARS];
  int wield[2];
  struct obj_data *temp, *next_obj;

  wield[0] = GET_WIELDED(ch, 0);
  wield[1] = GET_WIELDED(ch, 1);

  if (IS_NPC(ch) || GET_PFILEPOS(ch) < 0)
   return;

  /* This version of save_char allows ch->desc to be null (to allow
     "set file" to use it).  This causes the player's last host
     and probably last login to null out.
  */


   printf("Saving %s\r\n",GET_NAME(ch));

   for (i = 0;(*(bits + i) = LOWER(*(GET_NAME(ch) + i))); i++);
    sprintf(outname, "%s%s%c%s%s%s", PLR_PREFIX_N, SLASH , *bits, SLASH, bits, PLR_SUFFIX);

    if (!(fl = fopen(outname, "w+b")))
    {
      sprintf(buf, "SYSERR: Couldn't open player file %s for write", outname);
      printf(buf);
      return;
    }

    /* remove affects from eq and spells (from char_to_store) */
    /* Unaffect everything a character can be affected by */
  	GET_WIELDED(ch, 0) = wield[0];
  	GET_WIELDED(ch, 1) = wield[1];

    for (aff = ch->affected, i = 0; i < MAX_AFFECT; i++)
    {
      if (aff) {
        tmp_aff[i] = *aff;
        tmp_aff[i].next = 0;
        aff = aff->next;
      } else {
        tmp_aff[i].type = 0;	/* Zero signifies not used */
        tmp_aff[i].duration = 0;
        tmp_aff[i].modifier = 0;
        tmp_aff[i].location = 0;
        tmp_aff[i].bitvector = 0;
        tmp_aff[i].next = 0;
      }
    }

    /*
     * remove the affections so that the raw values are stored; otherwise the
     * effects are doubled when the char logs back in.
     */

    ch->aff_abils = ch->real_abils;

    /* end char_to_store code */

    /***************************************/
    /* Code Here for the actual Save Washu */
    /***************************************/

    /* Player Data */

    fprintf(fl, "Name: %s\n", GET_NAME(ch));
    fprintf(fl, "Pass: %s\n", GET_PASSWD(ch));
    if(ch->player.description && *ch->player.description)
    {
      strcpy(buf, ch->player.description);
      kill_ems(buf);
      fprintf(fl, "Desc: \n%s~\n", buf);
    }
    else
      fprintf(fl, "Desc: \nA %s stands here.\n~\n", pc_race_types[GET_RACE(ch)]);

    if(POOFIN(ch))
      fprintf(fl, "PfIn: %s\n", POOFIN(ch));
    else
      fprintf(fl, "PfIn: %s appears in a cloud of smoke.\n", GET_NAME(ch));

    if(POOFOUT(ch))
      fprintf(fl, "PfOt: %s\n", POOFOUT(ch));
    else
      fprintf(fl, "PfOt: %s disappears in a cloud of smoke.\n", GET_NAME(ch));

    if(GET_TITLE(ch))
      fprintf(fl, "Titl: %s\n", GET_TITLE(ch));
    if(GET_PRETITLE(ch))
      fprintf(fl, "PTit: %s\n", GET_PRETITLE(ch));
    if(GET_WHOTITLE(ch))
      fprintf(fl, "WTit: %s\n", GET_WHOTITLE(ch));
    else
      fprintf(fl, "WTit: %s\n", pc_race_types[GET_RACE(ch)]);
    if (ch->player.prompt)
  	  fprintf(fl, "Prom: %s\n", ch->player.prompt);

  	if(ch->player.mname)
      fprintf(fl, "Mnam: %s\n", ch->player.mname);
  	else
  	  fprintf(fl, "Mnam: persona\n");

  	if(ch->player.msdesc)
      fprintf(fl, "Msde: %s\n",ch->player.msdesc);
    else
      fprintf(fl, "Msde: a persona\n");
 	if (ch->player.mdesc)
      fprintf(fl, "Mdes: %s\n",ch->player.mdesc);
  	else
  	  fprintf(fl, "Mdes: A nondescript persona stands idly here.\n");
    if (ch->player.mldesc)
      fprintf(fl, "Mlde: %s\n", ch->player.mldesc);

    if (ch->player.aname)
      fprintf(fl, "Anam: %s\n", ch->player.aname);
    else
     fprintf(fl, "Anam: reflection\n");

    if (ch->player.asdesc)
      fprintf(fl, "Asde: %s\n", ch->player.asdesc);
    else
      fprintf(fl, "Asde: a reflection\n");

    if (ch->player.adesc)
      fprintf(fl, "Ades: %s\n", ch->player.adesc);
    else
      fprintf(fl, "Ades: The reflection of some physical being stands here.\n");

    if (ch->player.aldesc)
      fprintf(fl, "Alde: %s\n", ch->player.aldesc);

    fprintf(fl, "Sex : %d\n", GET_SEX(ch));
    fprintf(fl, "Levl: %d\n", GET_LEVEL(ch));
    fprintf(fl, "Home: %d\n", NOWHERE);
    fprintf(fl, "Brth: %ld\n", ch->player.time.birth);
    fprintf(fl, "Plyd: %d\n", ch->player.time.played);
    fprintf(fl, "Last: %ld\n", ch->player.time.logon);
    //if(ch->player.host)
    //	fprintf(fl, "Host: %s\n", ch->player.host);
    fprintf(fl, "Hite: %d\n", GET_HEIGHT(ch));
    fprintf(fl, "Wate: %d\n", GET_WEIGHT(ch));
    fprintf(fl, "Trad: %d\n", GET_TRADITION(ch));
	fprintf(fl, "Race: %d\n", GET_RACE(ch));
    /* End Player Data */
	/* Ability Data*/

    fprintf(fl, "Str : %d\n", GET_STR(ch));
    fprintf(fl, "Int : %d\n", GET_INT(ch));
    fprintf(fl, "Wil : %d\n", GET_WIL(ch));
    fprintf(fl, "Qui : %d\n", GET_QUI(ch));
    fprintf(fl, "Bod : %d\n", GET_BOD(ch));
    fprintf(fl, "Cha : %d\n", GET_CHA(ch));
	fprintf(fl, "Mag : %d\n", GET_MAG(ch));
	fprintf(fl, "RMag: 6\n");
	fprintf(fl, "Ess : %d\n", 6);
	fprintf(fl, "Rea : %d\n", 0);
	fprintf(fl, "BInd: %d\n", GET_INDEX(ch));
	fprintf(fl, "Apoo: %d\n", 0);
	fprintf(fl, "Dpoo: %d\n", 0);
	fprintf(fl, "Cpoo: %d\n", 0);
	fprintf(fl, "Hpoo: %d\n", 0);
	fprintf(fl, "Mpoo: %d\n", 0);
	fprintf(fl, "Opoo: %d\n", 0);
	fprintf(fl, "Rpoo: 0\n");
	/* End Ability data */

	/* Point Data */
	fprintf(fl, "Balc: %d\n",0);
	fprintf(fl, "Impt: %d\n",0);
	fprintf(fl, "IDic: %d\n",0);
	fprintf(fl, "IRol: %d\n",0);
	fprintf(fl, "Sust: %d\n",0);
    fprintf(fl, "Ment: %d/%d\n", GET_MENTAL(ch), GET_MAX_MENTAL(ch));
    fprintf(fl, "Phys: %d/%d\n", GET_PHYSICAL(ch), GET_MAX_PHYSICAL(ch));
    fprintf(fl, "Nuye: %d\n", GET_NUYEN(ch));
    fprintf(fl, "Bank: %d\n", GET_BANK(ch));
    fprintf(fl, "Karm: %d\n", GET_KARMA(ch));
    fprintf(fl, "Rep : %d\n", GET_REP(ch));
	/* End point data */
	/* Char Specials Saved */

    fprintf(fl, "Alin: %d\n", GET_ALIGNMENT(ch));
    fprintf(fl, "Id  : %ld\n",  GET_IDNUM(ch));
    if(PLR_FLAGS(ch) != 0)
      fprintf(fl, "Act : %ld\n", PLR_FLAGS(ch));
    if(AFF_FLAGS(ch) != 0)
    {
      sprintbits(AFF_FLAGS(ch), bits);
      fprintf(fl, "Aff : %s\n", bits);
    }
    fprintf(fl, "Skil:\n");
    for(i = 1; i <= MAX_SKILLS; i++)
    {
	  if(GET_SKILL(ch, i))
		fprintf(fl, "%d %d\n", i, GET_SKILL(ch, i));
    }
    fprintf(fl, "0 0\n");
	fprintf(fl, "LHan: %d\n",ch->char_specials.saved.left_handed);
	fprintf(fl, "CLan: %d\n",0);

	/* End Char Special Saved*/

	/* player Specials Saved */

    fprintf(fl, "Wimp: %d\n", GET_WIMP_LEV(ch));
    if(GET_FREEZE_LEV(ch) != 0)
      fprintf(fl, "Frez: %d\n", GET_FREEZE_LEV(ch));
    if(GET_INVIS_LEV(ch) != 0)
      fprintf(fl, "Invs: %d\n", GET_INVIS_LEV(ch));
    //if(GET_LOADROOM(ch) != 0)
      fprintf(fl, "Room: %d\n", load_room);
    if(PRF_FLAGS(ch) != 0)
    {
      sprintbits(PRF_FLAGS(ch), bits);
      fprintf(fl, "Pref: %s\n", bits);
    }
    if(GET_BAD_PWS(ch) != 0)
      fprintf(fl, "Badp: %d\n", GET_BAD_PWS(ch));
    if(GET_COND(ch, FULL) != 0 && GET_LEVEL(ch) < LVL_LEGEND)
      fprintf(fl, "Hung: %d\n", GET_COND(ch, FULL));
    if(GET_COND(ch, THIRST) != 0 && GET_LEVEL(ch) < LVL_LEGEND)
      fprintf(fl, "Thir: %d\n", GET_COND(ch, THIRST));
    if(GET_COND(ch, DRUNK) != 0 && GET_LEVEL(ch) < LVL_LEGEND)
      fprintf(fl, "Drnk: %d\n", GET_COND(ch, DRUNK));

	if(GET_ALLERGY(ch) != ALLERGIC_NONE)
	{
	  fprintf(fl, "Ally: %d\n", GET_ALLERGY(ch));
	  fprintf(fl, "Asev: %d\n", GET_SEVERITY(ch));
	  fprintf(fl, "Arec: %d\n", 0);
	}

	if(GET_TOTEM(ch) != 0)
	  fprintf(fl, "Totm: %d\n", GET_TOTEM(ch));
	if(GET_ATT_POINTS(ch) != 0)
	  fprintf(fl, "Satt: %d\n", GET_ATT_POINTS(ch));
	if(GET_SKILL_POINTS(ch) != 0)
	  fprintf(fl, "Sskl: %d\n", GET_SKILL_POINTS(ch));
	if(GET_LEVEL(ch) >= LVL_LEGEND)
	  fprintf(fl, "Znum: %d\n", ch->player_specials->saved.zonenum);

	/* End Player Specials */

    /* affected_type */
    if(tmp_aff[0].type > 0)
    {
      fprintf(fl, "Affs:\n");
      for(i = 0; i < MAX_AFFECT; i++)
      {
		aff = &tmp_aff[i];
		if(aff->type)
	  		fprintf(fl, "%d %d %d %d %d\n", aff->type, aff->duration,
	    aff->modifier, aff->location, (int)aff->bitvector);
      }
      fprintf(fl, "0 0 0 0 0\n");
    }

    fclose(fl);

    /* more char_to_store code to restore affects */


    //if((id = find_name(GET_NAME(ch))) < 0)
      //return;
    /*
    i = player_table[id].flags;
    if(PLR_FLAGGED(ch, PLR_DELETED))
       SET_BIT(player_table[id].flags, PINDEX_DELETED);
    else
       REMOVE_BIT(player_table[id].flags, PINDEX_DELETED);
    if(PLR_FLAGGED(ch, PLR_NODELETE) || PLR_FLAGGED(ch, PLR_CRYO))
       SET_BIT(player_table[id].flags, PINDEX_NODELETE);
    else
      REMOVE_BIT(player_table[id].flags, PINDEX_NODELETE);
    if(player_table[id].flags != i || save_index)
      save_player_index();

    */
    /* update the player in the player index */
    //save_player_index();
}

//int find_name(char *name)
//{
//  int i;
//
//  for (i = 0; i <= top_of_p_table; i++) {
//    if (!str_cmp((player_table + i)->name, name))
//      return i;
//  }
//
//  return -1;
//}

void kill_ems(char *str)
{
  char *ptr1, *ptr2, *tmp;

  tmp = str;
  ptr1 = str;
  ptr2 = str;

  while(*ptr1) {
    if((*(ptr2++) = *(ptr1++)) == '\r')
      if(*ptr1 == '\r')
	ptr1++;
  }
  *ptr2 = '\0';
}

void store_to_char(struct char_file_u * st, struct char_data * ch)
{
  /* to save memory, only PC's -- not MOB's -- have player_specials */
  if (ch->player_specials == NULL)
    ch->player_specials = new player_special_data;

  GET_SEX(ch) = st->sex;
  GET_RACE(ch) = st->race;
  GET_LEVEL(ch) = st->level;
  GET_TRADITION(ch) = st->tradition;
  if (GET_TRADITION(ch) != TRAD_HERMETIC && GET_TRADITION(ch) != TRAD_SHAMANIC &&
      GET_TRADITION(ch) != TRAD_ADEPT && GET_TRADITION(ch) != TRAD_MUNDANE)
    GET_TRADITION(ch) = TRAD_MUNDANE;

  ch->player.name = new char[strlen(st->name) + 1];
  strcpy(ch->player.name, st->name);
  strcpy(ch->player.passwd, st->pwd);
  ch->player.short_descr = NULL;
  ch->player.long_descr = NULL;
  ch->player.title = str_dup(st->title);
  ch->player.pretitle = str_dup(st->pretitle);
  ch->player.whotitle = str_dup(st->whotitle);
  ch->player.description = str_dup(st->description);
  ch->player.prompt = str_dup(st->prompt);
  ch->player.poofin = str_dup(st->poofin);
  ch->player.poofout = str_dup(st->poofout);
  ch->player.mname = str_dup(st->mname);
  ch->player.msdesc = str_dup(st->msdesc);
  ch->player.mdesc = str_dup(st->mdesc);
  ch->player.mldesc = str_dup(st->mldesc);
  ch->player.aname = str_dup(st->aname);
  ch->player.asdesc = str_dup(st->asdesc);
  ch->player.adesc = str_dup(st->adesc);
  ch->player.aldesc = str_dup(st->aldesc);

  ch->player.last_room = st->hometown;
  ch->player.time.birth = st->birth;
  ch->player.time.played = st->played;
  ch->player.time.logon = time(0);

  ch->player.weight = st->weight;
  ch->player.height = st->height;

  ch->real_abils = st->abilities;
  ch->aff_abils = st->abilities;
  ch->points = st->points;
  ch->char_specials.saved = st->char_specials_saved;
  ch->player_specials->saved = st->player_specials_saved;

  //if (ch->points.max_physical < 1000)
  ch->points.max_physical = 1000;

  //if (ch->points.max_mental < 1000)
  ch->points.max_mental = 1000;

  ch->char_specials.carry_weight = 0;
  ch->char_specials.carry_items = 0;
  ch->points.ballistic = 0;
  ch->points.init_dice = 0;
  ch->points.init_roll = 0;
  ch->points.sustained = 0;


  //ch->in_room = real_room(GET_LOADROOM(ch));
  GET_LAST_TELL(ch) = NOBODY;


  /*
   * If you're not poisioned and you've been away for more than an hour,
   * we'll set your HMV back to full
   */

  //if (!IS_AFFECTED(ch, AFF_POISON) &&
  //    (((long) (time(0) - st->last_logon)) >= SECS_PER_REAL_HOUR)) {
  //  GET_PHYSICAL(ch) = GET_MAX_PHYSICAL(ch);
  //  GET_MENTAL(ch) = GET_MAX_MENTAL(ch);
  //}
}                               /* store_to_char */

char *str_dup(const char *source)
{
  char *New = new char[strlen(source) + 1];
  return (strcpy(New, source));
}
/*thanks to Luis Carvalho for sending this my way..it's probably a
  lot shorter than the one I would have made :)  */
void sprintbits(long vektor,char *outstring)
{
  int i;
  char flags[53]="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

  strcpy(outstring,"");
  for (i=0;i<32;i++)
  {
    if (vektor & 1) {
      *outstring=flags[i];
      outstring++;
    };
    vektor>>=1;
  };
  *outstring=0;
}
